import tkinter
from datetime import date
import pickle
import os

class BirthdayReminder(tkinter.Frame):
    def __init__(self, master=None):
        super().__init__(master, padx=10, pady=20)
        self.pack()

        self.birthday = None
        filename = "birthday.pickle"
        self.file_path = os.path.join(os.path.dirname(__file__),
                filename)

        # 誕生日を表示/設定するフレーム
        self.birthday_frame = tkinter.Frame(self)
        self.birthday_frame.pack(pady=10)

        # 年
        self.birthday_year = tkinter.IntVar()
        self.year_entry = tkinter.Entry(
            self.birthday_frame, textvariable=self.birthday_year,
            width=4, justify=tkinter.RIGHT)
        self.year_entry.pack(side=tkinter.LEFT)
        tkinter.Label(self.birthday_frame,
                text="年").pack(side=tkinter.LEFT)

        # 月
        self.birthday_month = tkinter.IntVar()
        self.month_entry = tkinter.Entry(
            self.birthday_frame, textvariable=self.birthday_month,
            width=2, justify=tkinter.RIGHT)
        self.month_entry.pack(side=tkinter.LEFT)
        tkinter.Label(self.birthday_frame,
                text="月").pack(side=tkinter.LEFT)

        # 日
        self.birthday_day = tkinter.IntVar()
        self.day_entry = tkinter.Entry(
            self.birthday_frame, textvariable=self.birthday_day,
            width=2, justify=tkinter.RIGHT)
        self.day_entry.pack(side=tkinter.LEFT)
        tkinter.Label(
            self.birthday_frame,
            text="日生まれ").pack(side=tkinter.LEFT)

        # 保存ボタン
        self.save_btn = tkinter.Button(
            self.birthday_frame, text="保存", command=self.save)
        self.save_btn.pack(side=tkinter.LEFT, padx=5)

        # 残り日数/年齢表示用フレーム
        self.disp_frame = tkinter.Frame(self)
        self.disp_frame.pack()
        self.day_left_label = tkinter.Label(self.disp_frame)
        self.day_left_label.pack()

        self.age_label = tkinter.Label(self.disp_frame)
        self.age_label.pack()

        # birthday.pickleが存在していたら誕生日を読み込む
        if os.path.exists(self.file_path):
            in_file = open(self.file_path, "rb")
            self.birthday = pickle.load(in_file)
            self.birthday_year.set(self.birthday.year)
            self.birthday_month.set(self.birthday.month)
            self.birthday_day.set(self.birthday.day)

        # birthdayが設定されていればcheck()メソッドを実行
        if self.birthday is not None:
            self.check()

    def save(self):
        self.birthday = date(self.birthday_year.get(),
                             self.birthday_month.get(),
                             self.birthday_day.get())
        out_file = open(self.file_path, "wb")
        pickle.dump(self.birthday, out_file)
        self.check()

    def check(self):
        today = date.today()
        # 年齢を計算
        age = today.year - self.birthday.year
        # 今年の誕生日
        birthday2 = date(today.year, self.birthday.month,
            self.birthday.day)
        if birthday2 == today:
            self.day_left_label["text"] = "誕生日おめでとう"
        elif birthday2 < today:
            # 誕生日がすぎていれば来年の誕生日に
            birthday2 = date(birthday2.year + 1,
                             birthday2.month, birthday2.day)
            diff = birthday2 - today
            self.day_left_label["text"] = "誕生日まであと" + str(diff.days) + "日"
        else:
            age -= 1
            diff = birthday2 - today
            self.day_left_label["text"] = "誕生日まであと" + str(diff.days) + "日"
        self.age_label["text"] = str(age) + "才"

if __name__ == '__main__':
    root = tkinter.Tk()
    root.title("バースディリマインダー")
    root.option_add("*font", "Courier 24")
    app = BirthdayReminder(master=root)
    app.mainloop()
