import tkinter

class Oekaki(tkinter.Frame):
    def __init__(self, master=None):
        super().__init__(master)
        self.pack()

        # キャンバスサイズ
        self.canvas_width = 900
        self.canvas_height = 500

        # 線の色
        self.color = "black"
        # 線幅
        self.line_width = 3

        # マウスの位置
        #self.px = 0
        #self.py = 0
        # 最後に書いた線
        self.last_line = []
        # 全ての線
        self.all_lines = []
        # キャンバス
        self.canvas = tkinter.Canvas(
            self, width=self.canvas_width, height=self.canvas_height)
        self.canvas.pack()
        self.canvas.bind("<ButtonPress-1>", self.button_pressed)
        self.canvas.bind("<B1-Motion>", self.mouse_moved)
        self.canvas.bind("<ButtonRelease-1>", self.button_up)

        # ツールバー
        self.create_toolbar()

    def create_toolbar(self):
        # ツールバー
        self.toolbar = tkinter.Frame(self)
        self.toolbar.pack()
        # 色
        self.color_sel = tkinter.StringVar()
        self.color_sel.set(self.color)
        black = tkinter.Radiobutton(self.toolbar, text="黒 ",
                        variable=self.color_sel, value="black")
        black.pack(side=tkinter.LEFT)
        red = tkinter.Radiobutton(self.toolbar, text="赤 ",
                        variable=self.color_sel, value="red")
        red.pack(side=tkinter.LEFT)
        green = tkinter.Radiobutton(self.toolbar, text="緑 ",
                        variable=self.color_sel, value="green")
        green.pack(side=tkinter.LEFT)
        yellow = tkinter.Radiobutton(self.toolbar, text="黄 ",
                        variable=self.color_sel, value="yellow")
        yellow.pack(side=tkinter.LEFT)

        # 線幅
        self.line_var = tkinter.IntVar()
        self.line_var.set(self.line_width)
        line_ws = (1, 2, 3, 4, 5, 6, 7, 8)
        line_menu = tkinter.OptionMenu(self.toolbar,
                    self.line_var, *line_ws)
        line_menu.pack(side=tkinter.LEFT)

        # 取り消し
        undo_btn = tkinter.Button(self.toolbar, text="取り消し",
                                  command=self.undo)
        undo_btn.pack(side=tkinter.LEFT, padx=15)


    def button_pressed(self, event):
        self.px = event.x
        self.py = event.y
        # 線の色
        self.color = self.color_sel.get()
        # 線幅
        self.line_width = self.line_var.get()

    def mouse_moved(self, event):
        x = event.x
        y = event.y
        id = self.canvas.create_line(self.px, self.py, x, y,
                            fill=self.color, width=self.line_width)
        self.px = x
        self.py = y
        self.last_line.append(id)

    def button_up(self, event):
        self.all_lines.append(self.last_line)
        self.last_line = []

    def undo(self):
        if len(self.all_lines) > 0:
            last = self.all_lines.pop()
            for l in last:
                self.canvas.delete(l)


if __name__ == '__main__':
    root = tkinter.Tk()
    root.option_add("*font", "Courier 30")
    root.title("お絵かき")
    Oekaki(master=root)
    root.mainloop()
