import tkinter
import random
import os

class Puzzle15(tkinter.Frame):

    def __init__(self, master=None):
        super().__init__(master)
        self.pack()
        self.empty = 15  # 空のピースの場所
        self.pieces = []  # ピースを管理するリスト

        # イメージの読み込み
        self.images = []
        dir_path = os.path.join(os.path.dirname(__file__) , "images")
        for n in range(0, 15):
            file_path =  os.path.join(dir_path,str(n) + ".gif")
            self.images.append(tkinter.PhotoImage(file=file_path))

        # 「シャッフル」ボタン用フレーム
        btn_frame = tkinter.Frame(self)
        btn = tkinter.Button(btn_frame, text="シャッフル",
                                command=self.shuffle)
        btn.pack()
        btn_frame.pack()

        # ピース配置用フレーム
        board_frame = tkinter.Frame(self)
        board_frame.pack()
        # ピースを配置
        for pos in range(0, 15):
            piece = tkinter.Label(board_frame, image=self.images[pos])
            piece.grid(row=int(pos / 4), column=pos % 4)
            # numプロパティにピースの位置を代入
            piece.num = pos
            piece.bind("<Button-1>", self.piece_clicked)
            self.pieces.append(piece)   # ピースを追加

    def shuffle(self):
        for i in range(1, 100):
            pos = random.randint(0, 14)
            self.pieces[pos].event_generate("<Button-1>")

    def piece_clicked(self, event):
        piece = event.widget
        if self.is_movable(piece):
            # 空白とピースを入れ替える
            tmp = self.empty
            self.empty = piece.num
            piece.num = tmp
            piece.grid(row=int(tmp / 4), column=tmp % 4)

    def is_movable(self, piece):
        diff = piece.num - self.empty
        if abs(diff) == 4:
            return True
        if diff == 1 and (self.empty % 4 != 3):
            return True
        if diff == -1 and (self.empty % 4 != 0):
            return True
        return False

if __name__ == '__main__':
    root = tkinter.Tk()
    root.title("15パズル")
    puzzle = Puzzle15(master=root)
    puzzle.mainloop()
