from datetime import date
import calendar
import tkinter


class MyCalendar(tkinter.Frame):
    def __init__(self, master=None):
        super().__init__(master)
        self.pack()

        # 今日の年と月
        today = date.today()
        self.year = today.year
        self.month = today.month
        # 今月のカレンダーを表示
        self.label_frame = tkinter.Frame(self)
        self.label_frame.pack()
        # 年と月とボタンを表示
        self.show_yearmonth(self.year, self.month)
        # カレンダーの本体を表示
        self.cal_frame = tkinter.Frame(self)
        self.cal_frame.pack()
        self.show_calendar(self.year, self.month)

    def to_this_month(self, event):
        today = date.today()
        self.year = today.year
        self.month = today.month
        self.renew_cal()

    def next_month(self):
        self.month += 1
        if self.month == 13:
            self.month = 1
            self.year += 1
        self.renew_cal()

    def prev_month(self):
        self.month -= 1
        if self.month == 0:
            self.month = 12
            self.year -= 1
        self.renew_cal()

    def renew_cal(self):
        """ カレンダーを更新する """
        self.cal_frame.destroy()  # クリア
        self.cal_frame = tkinter.Frame(self)
        self.ym_label["text"]  = str(self.year) + "年" + str(self.month) +"月"
        self.show_calendar(self.year, self.month)
        self.cal_frame.pack()

    def show_yearmonth(self, year, month):
        """  カレンダーの年月とボタンを表示 """
        self.prev_btn = tkinter.Button(self.label_frame, text="<",
                                       command=self.prev_month)
        self.prev_btn.pack(side=tkinter.LEFT)

        self.ym_label = tkinter.Label(self.label_frame, width=10,
                text=str(self.year) + "年" + str(self.month) + "月")
        #　ダブルクリックで今月へ
        self.ym_label.bind("<Double-Button-1>", self.to_this_month)
        self.ym_label.pack(side=tkinter.LEFT)
        self.next_btn = tkinter.Button(self.label_frame, text=">",
                                       command=self.next_month)
        self.next_btn.pack()

    def show_calendar(self, year, month):
        """ カレンダーの日付部分を表示 """
        cal = calendar.Calendar()
        month_days = cal.monthdayscalendar(year, month)
        weekdays = ["月", "火", "水", "木", "金", "土", "日"]
        # 曜日を表示
        for wd in range(0, 7):
            w = tkinter.Label(self.cal_frame, text=weekdays[wd])
            w.grid(column=wd, row=0, sticky=tkinter.E)
        # 日を表示
        for row in range(0, len(month_days)):
            week_days = month_days[row]
            for col in range(0, 7):
                if week_days[col] != 0:
                    d = tkinter.Label(self.cal_frame, text=week_days[col])
                    d.grid(column=col, row=row + 1, sticky=tkinter.E)


if __name__ == '__main__':
    root = tkinter.Tk()
    root.option_add("*font", "Courier 30")
    root.geometry("400x400")
    root.title("カレンダー")
    my_cal = MyCalendar(master=root)
    root.mainloop()
